import * as ts from 'typescript';
import * as ng from '@angular/language-service';
export interface Logger {
    close(): void;
    isVerbose(): boolean;
    info(s: string): void;
    startGroup(): void;
    endGroup(): void;
    msg(s: string, type?: string): void;
}
export interface ProjectServiceHost {
    getCurrentDirectory(): string;
    readFile(path: string, encoding?: string): string;
    directoryExists(path: string): boolean;
    getExecutingFilePath(): string;
    resolvePath(path: string): string;
    fileExists(path: string): boolean;
    getDirectories(path: string): string[];
    watchDirectory(path: string, callback: ts.DirectoryWatcherCallback, recursive?: boolean): ts.FileWatcher;
    watchFile(path: string, callback: ts.FileWatcherCallback): ts.FileWatcher;
    readDirectory(path: string, extensions?: string[], exclude?: string[], include?: string[]): string[];
    useCaseSensitiveFileNames: boolean;
    newLine?: string;
    setTimeout(callback: (...args: any[]) => void, ms: number, ...args: any[]): any;
    clearTimeout(timeoutId: any): void;
}
export declare class ScriptInfo {
    private host;
    fileName: string;
    isOpen: boolean;
    svc: ScriptVersionCache;
    children: ScriptInfo[];
    defaultProject: Project;
    fileWatcher: ts.FileWatcher;
    path: ts.Path;
    scriptKind: ts.ScriptKind;
    constructor(host: ProjectServiceHost, fileName: string, content: string, isOpen?: boolean);
    close(): void;
    addChild(childInfo: ScriptInfo): void;
    snap(): LineIndexSnapshot;
    getText(): string;
    getLineInfo(line: number): ILineInfo;
    editContent(start: number, end: number, newText: string): void;
    getTextChangeRangeBetweenVersions(startVersion: number, endVersion: number): ts.TextChangeRange;
    getChangeRange(oldSnapshot: ts.IScriptSnapshot): ts.TextChangeRange;
}
export declare class LSHost implements ts.LanguageServiceHost {
    host: ProjectServiceHost;
    project: Project;
    ls: ts.LanguageService;
    compilationSettings: ts.CompilerOptions;
    filenameToScript: Map<ts.Path, ScriptInfo>;
    roots: ScriptInfo[];
    version: number;
    private resolvedModuleNames;
    private resolvedTypeReferenceDirectives;
    private moduleResolutionHost;
    private getCanonicalFileName;
    constructor(host: ProjectServiceHost, project: Project);
    private resolveNamesWithLocalCache;
    getProjectVersion(): string;
    resolveTypeReferenceDirectives(typeDirectiveNames: string[], containingFile: string): ts.ResolvedTypeReferenceDirective[];
    resolveModuleNames(moduleNames: string[], containingFile: string): ts.ResolvedModule[];
    getDefaultLibFileName(): string;
    getScriptSnapshot(filename: string): ts.IScriptSnapshot;
    setCompilationSettings(opt: ts.CompilerOptions): void;
    lineAffectsRefs(filename: string, line: number): boolean;
    getCompilationSettings(): ts.CompilerOptions;
    getScriptFileNames(): string[];
    getScriptKind(fileName: string): ts.ScriptKind;
    getScriptVersion(filename: string): string;
    getCurrentDirectory(): string;
    getScriptIsOpen(filename: string): boolean;
    removeReferencedFile(info: ScriptInfo): void;
    getScriptInfo(filename: string): ScriptInfo;
    addRoot(info: ScriptInfo): void;
    removeRoot(info: ScriptInfo): void;
    reloadScript(filename: string, tmpfilename: string, cb: () => any): void;
    editScript(filename: string, start: number, end: number, newText: string): void;
    resolvePath(path: string): string;
    fileExists(path: string): boolean;
    directoryExists(path: string): boolean;
    getDirectories(path: string): string[];
    /**
     *  @param line 1 based index
     */
    lineToTextSpan(filename: string, line: number): ts.TextSpan;
    /**
     * @param line 1 based index
     * @param offset 1 based index
     */
    lineOffsetToPosition(filename: string, line: number, offset: number): number;
    /**
     * @param line 1-based index
     * @param offset 1-based index
     */
    positionToLineOffset(filename: string, position: number, lineIndex?: LineIndex): ILineInfo;
    getLineIndex(filename: string): LineIndex;
    private modified;
}
export interface ProjectOptions {
    files?: string[];
    wildcardDirectories?: ts.MapLike<ts.WatchDirectoryFlags>;
    compilerOptions?: ts.CompilerOptions;
}
export declare class Project {
    projectService: ProjectService;
    private logger;
    private documentRegistry;
    projectOptions?: ProjectOptions;
    languageServiceDisabled: boolean;
    compilerService: CompilerService;
    projectFilename: string;
    projectFileWatcher: ts.FileWatcher;
    directoryWatcher: ts.FileWatcher;
    directoriesWatchedForWildcards: ts.Map<ts.FileWatcher>;
    directoriesWatchedForTsconfig: string[];
    program: ts.Program;
    filenameToSourceFile: ts.Map<ts.SourceFile>;
    referencesFile: Set<string>;
    updateGraphSeq: number;
    /** Used for configured projects which may have multiple open roots */
    openRefCount: number;
    constructor(projectService: ProjectService, logger: Logger, documentRegistry: ts.DocumentRegistry, projectOptions?: ProjectOptions, languageServiceDisabled?: boolean);
    enableLanguageService(): void;
    disableLanguageService(): void;
    addOpenRef(): void;
    deleteOpenRef(): number;
    openReferencedFile(filename: string): any;
    getRootFiles(): string[];
    getFileNames(): string[];
    references(info: ScriptInfo): boolean;
    getSourceFile(info: ScriptInfo): any;
    getSourceFileFromName(filename: string, requireOpen?: boolean): any;
    isRoot(info: ScriptInfo): boolean;
    removeReferencedFile(info: ScriptInfo): void;
    updateFileMap(): void;
    finishGraph(): void;
    updateGraph(): void;
    isConfiguredProject(): string;
    addRoot(info: ScriptInfo): void;
    removeRoot(info: ScriptInfo): void;
    filesToString(): string;
    setProjectOptions(projectOptions: ProjectOptions): void;
}
export interface ProjectOpenResult {
    success?: boolean;
    errorMsg?: string;
    project?: Project;
}
/**
 * This helper funciton processes a list of projects and return the concatenated, sortd and deduplicated output of processing each project.
 */
export declare function combineProjectOutput<T>(projects: Project[], action: (project: Project) => T[], comparer?: (a: T, b: T) => number, areEqual?: (a: T, b: T) => boolean): T[];
export declare type ProjectServiceEvent = {
    eventName: "context" | "change" | "opened" | "closed";
    data: {
        project: Project;
        fileName: string;
    };
} | {
    eventName: "configFileDiag";
    data: {
        triggerFile?: string;
        configFileName: string;
        diagnostics: ts.Diagnostic[];
    };
};
export interface ProjectServiceEventHandler {
    (event: ProjectServiceEvent): void;
}
export interface HostConfiguration {
    formatCodeOptions: ts.FormatCodeOptions;
    hostInfo: string;
}
export declare class ProjectService {
    host: ProjectServiceHost;
    psLogger: Logger;
    eventHandler?: ProjectServiceEventHandler;
    documentRegistry: ts.DocumentRegistry;
    filenameToScriptInfo: ts.Map<ScriptInfo>;
    openFileRoots: ScriptInfo[];
    inferredProjects: Project[];
    configuredProjects: Project[];
    openFilesReferenced: ScriptInfo[];
    openFileRootsConfigured: ScriptInfo[];
    directoryWatchersForTsconfig: ts.Map<ts.FileWatcher>;
    directoryWatchersRefCount: ts.Map<number>;
    hostConfiguration: HostConfiguration;
    timerForDetectingProjectFileListChanges: ts.Map<any>;
    changeSeq: number;
    constructor(host: ProjectServiceHost, psLogger: Logger, eventHandler?: ProjectServiceEventHandler);
    addDefaultHostConfiguration(): void;
    watchedFileChanged(fileName: string): void;
    private report;
    /**
     * This is the callback function when a watched directory has added or removed source code files.
     * @param project the project that associates with this directory watcher
     * @param fileName the absolute file name that changed in watched directory
     */
    directoryWatchedForSourceFilesChanged(project: Project, fileName: string): void;
    startTimerForDetectingProjectFileListChanges(project: Project): void;
    handleProjectFileListChanges(project: Project): void;
    reportConfigFileDiagnostics(configFileName: string, diagnostics: ts.Diagnostic[], triggerFile?: string): void;
    /**
     * This is the callback function when a watched directory has an added tsconfig file.
     */
    directoryWatchedForTsconfigChanged(fileName: string): void;
    getCanonicalFileName(fileName: string): string;
    watchedProjectConfigFileChanged(project: Project): void;
    log(msg: string, type?: string): void;
    closeLog(): void;
    createInferredProject(root: ScriptInfo): Project;
    fileDeletedInFilesystem(info: ScriptInfo): void;
    removeProject(project: Project): void;
    setConfiguredProjectRoot(info: ScriptInfo): boolean;
    addOpenFile(info: ScriptInfo): void;
    /**
      * Remove this file from the set of open, non-configured files.
      * @param info The file that has been closed or newly configured
      */
    closeOpenFile(info: ScriptInfo): void;
    findReferencingProjects(info: ScriptInfo, excludedProject?: Project): Project[];
    /**
     * This function rebuilds the project for every file opened by the client
     */
    reloadProjects(): void;
    /**
     * This function is to update the project structure for every projects.
     * It is called on the premise that all the configured projects are
     * up to date.
     */
    updateProjectStructure(): void;
    getScriptInfo(filename: string): any;
    /**
     * @param filename is absolute pathname
     * @param fileContent is a known version of the file content that is more up to date than the one on disk
     */
    openFile(fileName: string, openedByClient: boolean, fileContent?: string, scriptKind?: ts.ScriptKind): any;
    findConfigFile(searchPath: string): string;
    /**
     * Open file whose contents is managed by the client
     * @param filename is absolute pathname
     * @param fileContent is a known version of the file content that is more up to date than the one on disk
     */
    openClientFile(fileName: string, fileContent?: string, scriptKind?: ts.ScriptKind): {
        configFileName?: string;
        configFileErrors?: ts.Diagnostic[];
    };
    /**
     * Report a client file change.
     */
    clientFileChanges(fileName: string, changes: {
        start: number;
        end: number;
        insertText: string;
    }[]): void;
    reloadScript(filename: string): void;
    forcedGetProjectForFile(fileName: string): Project;
    lineOffsetsToPositions(fileName: string, positions: {
        line: number;
        col: number;
    }[]): number[];
    positionsToLineOffsets(fileName: string, offsets: number[]): {
        line: number;
        col: number;
    }[];
    positionToLineOffset(fileName: string, offset: number): ILineInfo;
    private requestUpdateProjectStructure;
    /**
     * This function tries to search for a tsconfig.json for the given file. If we found it,
     * we first detect if there is already a configured project created for it: if so, we re-read
     * the tsconfig file content and update the project; otherwise we create a new one.
     */
    openOrUpdateConfiguredProjectForFile(fileName: string): {
        configFileName?: string;
        configFileErrors?: ts.Diagnostic[];
    };
    /**
     * Close file whose contents is managed by the client
     * @param filename is absolute pathname
     */
    closeClientFile(filename: string): void;
    getProjectForFile(filename: string): any;
    printProjectsForFile(filename: string): void;
    printProjects(): void;
    configProjectIsActive(fileName: string): boolean;
    findConfiguredProjectByConfigFile(configFileName: string): Project;
    configFileToProjectOptions(configFilename: string): {
        projectOptions?: ProjectOptions;
        errors: ts.Diagnostic[];
    };
    openConfigFile(configFilename: string, clientFileName?: string): {
        project?: Project;
        errors: ts.Diagnostic[];
    };
    updateConfiguredProject(project: Project): ts.Diagnostic[];
    createProject(projectFilename: string, projectOptions?: ProjectOptions, languageServiceDisabled?: boolean): Project;
}
export declare class CompilerService {
    project: Project;
    private logger;
    documentRegistry: ts.DocumentRegistry;
    host: LSHost;
    languageService: ts.LanguageService;
    ngHost: ng.TypeScriptServiceHost & {
        setSite?: (ls: ng.LanguageService) => void;
    };
    ngService: ng.LanguageService;
    classifier: ts.Classifier;
    settings: ts.CompilerOptions;
    ng: typeof ng;
    constructor(project: Project, logger: Logger, documentRegistry: ts.DocumentRegistry, opt?: ts.CompilerOptions);
    setCompilerOptions(opt: ts.CompilerOptions): void;
    isExternalModule(filename: string): boolean;
    resolveLanguageServiceModule(): typeof ng;
    private log;
    static getDefaultFormatCodeOptions(host: ProjectServiceHost): ts.FormatCodeOptions;
}
export interface LineCollection {
    charCount(): number;
    lineCount(): number;
    isLeaf(): boolean;
    walk(rangeStart: number, rangeLength: number, walkFns: ILineIndexWalker): void;
}
export interface ILineInfo {
    line: number;
    offset: number;
    text?: string;
    leaf?: LineLeaf;
}
export declare enum CharRangeSection {
    PreStart = 0,
    Start = 1,
    Entire = 2,
    Mid = 3,
    End = 4,
    PostEnd = 5
}
export interface ILineIndexWalker {
    goSubtree: boolean;
    done: boolean;
    leaf(relativeStart: number, relativeLength: number, lineCollection: LineLeaf): void;
    pre?(relativeStart: number, relativeLength: number, lineCollection: LineCollection, parent: LineNode, nodeType: CharRangeSection): LineCollection;
    post?(relativeStart: number, relativeLength: number, lineCollection: LineCollection, parent: LineNode, nodeType: CharRangeSection): LineCollection;
}
export declare class TextChange {
    pos: number;
    deleteLen: number;
    insertedText?: string;
    constructor(pos: number, deleteLen: number, insertedText?: string);
    getTextChangeRange(): ts.TextChangeRange;
}
export declare class ScriptVersionCache {
    changes: TextChange[];
    versions: LineIndexSnapshot[];
    minVersion: number;
    private currentVersion;
    private host;
    static changeNumberThreshold: number;
    static changeLengthThreshold: number;
    static maxVersions: number;
    edit(pos: number, deleteLen: number, insertedText?: string): void;
    latest(): LineIndexSnapshot;
    latestVersion(): number;
    reloadFromFile(filename: string, cb?: () => any): void;
    reload(script: string): void;
    getSnapshot(): LineIndexSnapshot;
    getTextChangesBetweenVersions(oldVersion: number, newVersion: number): ts.TextChangeRange;
    static fromString(host: ProjectServiceHost, script: string): ScriptVersionCache;
}
export declare class LineIndexSnapshot implements ts.IScriptSnapshot {
    version: number;
    cache: ScriptVersionCache;
    index: LineIndex;
    changesSincePreviousVersion: TextChange[];
    constructor(version: number, cache: ScriptVersionCache);
    getText(rangeStart: number, rangeEnd: number): string;
    getLength(): number;
    getLineStartPositions(): number[];
    getLineMapper(): (line: number) => number;
    getTextChangeRangeSinceVersion(scriptVersion: number): ts.TextChangeRange;
    getChangeRange(oldSnapshot: ts.IScriptSnapshot): ts.TextChangeRange;
}
export declare class LineIndex {
    root: LineNode;
    checkEdits: boolean;
    charOffsetToLineNumberAndPos(charOffset: number): ILineInfo;
    lineNumberToInfo(lineNumber: number): ILineInfo;
    load(lines: string[]): void;
    walk(rangeStart: number, rangeLength: number, walkFns: ILineIndexWalker): void;
    getText(rangeStart: number, rangeLength: number): string;
    getLength(): number;
    every(f: (ll: LineLeaf, s: number, len: number) => boolean, rangeStart: number, rangeEnd?: number): boolean;
    edit(pos: number, deleteLength: number, newText?: string): LineIndex;
    static buildTreeFromBottom(nodes: LineCollection[]): LineNode;
    static linesFromText(text: string): {
        lines: string[];
        lineMap: number[];
    };
}
export declare class LineNode implements LineCollection {
    totalChars: number;
    totalLines: number;
    children: LineCollection[];
    isLeaf(): boolean;
    updateCounts(): void;
    execWalk(rangeStart: number, rangeLength: number, walkFns: ILineIndexWalker, childIndex: number, nodeType: CharRangeSection): boolean;
    skipChild(relativeStart: number, relativeLength: number, childIndex: number, walkFns: ILineIndexWalker, nodeType: CharRangeSection): void;
    walk(rangeStart: number, rangeLength: number, walkFns: ILineIndexWalker): void;
    charOffsetToLineNumberAndPos(lineNumber: number, charOffset: number): ILineInfo;
    lineNumberToInfo(lineNumber: number, charOffset: number): ILineInfo;
    childFromLineNumber(lineNumber: number, charOffset: number): {
        child: LineCollection;
        childIndex: number;
        relativeLineNumber: number;
        charOffset: number;
    };
    childFromCharOffset(lineNumber: number, charOffset: number): {
        child: LineCollection;
        childIndex: number;
        charOffset: number;
        lineNumber: number;
    };
    splitAfter(childIndex: number): LineNode;
    remove(child: LineCollection): void;
    findChildIndex(child: LineCollection): number;
    insertAt(child: LineCollection, nodes: LineCollection[]): LineNode[];
    add(collection: LineCollection): boolean;
    charCount(): number;
    lineCount(): number;
}
export declare class LineLeaf implements LineCollection {
    text: string;
    udata: any;
    constructor(text: string);
    setUdata(data: any): void;
    getUdata(): any;
    isLeaf(): boolean;
    walk(rangeStart: number, rangeLength: number, walkFns: ILineIndexWalker): void;
    charCount(): number;
    lineCount(): number;
}
