"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = __importStar(require("fs"));
var path = __importStar(require("path"));
var ts = __importStar(require("typescript/lib/tsserverlibrary")); // used as value
// NOTES:
// Be very careful about logging. There are two types of logging:
// 1. Console
// 2. File
// The language server could operate in a few different modes, depending on
// startup options. `process.argv` is parsed by `vscode-languageserver` when
// createConnection() is called.
// By default, the server communicates with the client through Node IPC.
// In this case, logging to stdout/stderr would be piped to VSCode's
// development console (Output tab in vscode).
// Verbose log should not be sent to the client as it could negatively impact
// performance. Instead, verbose log entries should be written to file using
// the Logger class here.
// The language server could also operate in JSON-RPC mode via stdin/stdout.
// In this case, there must not be any logging done through console.log(),
// console.info() etc, as it could pollute the communication channel.
// TLDR: To log to development console, always use connection.console.log().
// Never use console.log(), console.info(), etc directly.
/**
 * Create a logger instance to write to file.
 * @param options Logging options.
 */
function createLogger(options) {
    var logFile = options.get('logFile') ||
        path.join(fs.mkdtempSync('ng_'), 'ngserver.log');
    var logVerbosity = options.get('logVerbosity') || 'normal';
    var logLevel;
    switch (logVerbosity) {
        case 'terse':
            logLevel = ts.server.LogLevel.terse;
            break;
        case 'requestTime':
            logLevel = ts.server.LogLevel.requestTime;
            break;
        case 'verbose':
            logLevel = ts.server.LogLevel.verbose;
            break;
        case 'normal':
        default:
            logLevel = ts.server.LogLevel.terse;
            break;
    }
    return new Logger(logFile, false /* traceToConsole */, logLevel);
}
exports.createLogger = createLogger;
// TODO: Code below is from TypeScript's repository. Maybe create our own
// implementation.
// https://github.com/microsoft/TypeScript/blob/ec39d412876d0dcf704fc886d5036cb625220d2f/src/tsserver/server.ts#L120
function noop(_) { } // tslint:disable-line no-empty
function nowString() {
    // E.g. "12:34:56.789"
    var d = new Date();
    return d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds() + "." + d.getMilliseconds();
}
var Logger = /** @class */ (function () {
    function Logger(logFilename, traceToConsole, level) {
        this.logFilename = logFilename;
        this.traceToConsole = traceToConsole;
        this.level = level;
        this.fd = -1;
        this.seq = 0;
        this.inGroup = false;
        this.firstInGroup = true;
        if (this.logFilename) {
            try {
                this.fd = fs.openSync(this.logFilename, 'w');
            }
            catch (_a) {
                // swallow the error and keep logging disabled if file cannot be opened
            }
        }
    }
    Logger.padStringRight = function (str, padding) {
        return (str + padding).slice(0, padding.length);
    };
    Logger.prototype.close = function () {
        if (this.fd >= 0) {
            fs.close(this.fd, noop);
        }
    };
    Logger.prototype.getLogFileName = function () {
        return this.logFilename;
    };
    Logger.prototype.perftrc = function (s) {
        this.msg(s, ts.server.Msg.Perf);
    };
    Logger.prototype.info = function (s) {
        this.msg(s, ts.server.Msg.Info);
    };
    Logger.prototype.err = function (s) {
        this.msg(s, ts.server.Msg.Err);
    };
    Logger.prototype.startGroup = function () {
        this.inGroup = true;
        this.firstInGroup = true;
    };
    Logger.prototype.endGroup = function () {
        this.inGroup = false;
    };
    Logger.prototype.loggingEnabled = function () {
        return !!this.logFilename || this.traceToConsole;
    };
    Logger.prototype.hasLevel = function (level) {
        return this.loggingEnabled() && this.level >= level;
    };
    Logger.prototype.msg = function (s, type) {
        if (type === void 0) { type = ts.server.Msg.Err; }
        if (!this.canWrite)
            return;
        s = "[" + nowString() + "] " + s + "\n";
        if (!this.inGroup || this.firstInGroup) {
            var prefix = Logger.padStringRight(type + ' ' + this.seq.toString(), '          ');
            s = prefix + s;
        }
        this.write(s);
        if (!this.inGroup) {
            this.seq++;
        }
    };
    Object.defineProperty(Logger.prototype, "canWrite", {
        get: function () {
            return this.fd >= 0 || this.traceToConsole;
        },
        enumerable: true,
        configurable: true
    });
    Logger.prototype.write = function (s) {
        if (this.fd >= 0) {
            var buf = Buffer.from(s);
            // tslint:disable-next-line no-null-keyword
            fs.writeSync(this.fd, buf, 0, buf.length, /*position*/ null); // TODO: GH#18217
        }
        if (this.traceToConsole) {
            console.warn(s);
        }
    };
    return Logger;
}());
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map