"use strict";
/* --------------------------------------------------------------------------------------------
 * Portions Copyright (c) Microsoft Corporation. All rights reserved.
 * Portions Copyright (c) Google Inc. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var lsp = __importStar(require("vscode-languageserver"));
var documents_1 = require("./documents");
var errors_1 = require("./errors");
// Create a connection for the server. The connection uses Node's IPC as a transport
var connection = lsp.createConnection();
// Create a simple text document manager. The text document manager
// supports full document sync only
var documents = new documents_1.TextDocuments(handleTextEvent);
// Setup the error collector that watches for document events and requests errors
// reported back to the client
var errorCollector = new errors_1.ErrorCollector(documents, connection);
function handleTextEvent(event) {
    switch (event.kind) {
        case 'context':
        case 'change':
        case 'opened':
            errorCollector.requestErrors(event.document);
    }
}
// Make the text document manager listen on the connection
// for open, change and close text document events
documents.listen(connection);
// After the server has started the client sends an initilize request. The server receives
// in the passed params the rootPath of the workspace plus the client capabilites.
var workspaceRoot;
connection.onInitialize(function (params) {
    workspaceRoot = params.rootPath;
    return {
        capabilities: {
            // Tell the client that the server works in FULL text document sync mode
            textDocumentSync: documents.syncKind,
            // Tell the client that the server support code complete
            completionProvider: {
                resolveProvider: false,
                triggerCharacters: ['<', '.', '*', '[', '(']
            },
            definitionProvider: true,
            hoverProvider: true
        }
    };
});
function compiletionKindToCompletionItemKind(kind) {
    var CompletionItemKind = lsp.CompletionItemKind;
    switch (kind) {
        case 'attribute': return CompletionItemKind.Property;
        case 'html attribute': return CompletionItemKind.Property;
        case 'component': return CompletionItemKind.Class;
        case 'element': return CompletionItemKind.Class;
        case 'entity': return CompletionItemKind.Text;
        case 'key': return CompletionItemKind.Class;
        case 'method': return CompletionItemKind.Method;
        case 'pipe': return CompletionItemKind.Function;
        case 'property': return CompletionItemKind.Property;
        case 'type': return CompletionItemKind.Interface;
        case 'reference': return CompletionItemKind.Variable;
        case 'variable': return CompletionItemKind.Variable;
    }
    return CompletionItemKind.Text;
}
var wordRe = /(\w|\(|\)|\[|\]|\*|\-|\_|\.)+/g;
var special = /\(|\)|\[|\]|\*|\-|\_|\./;
// Convert attribute names with non-\w chracters into a text edit.
function insertionToEdit(range, insertText) {
    if (insertText.match(special) && range) {
        return lsp.TextEdit.replace(range, insertText);
    }
}
function getReplaceRange(document, offset) {
    var line = documents.getDocumentLine(document, offset);
    if (line && line.text && line.start <= offset && line.start + line.text.length >= offset) {
        var lineOffset_1 = offset - line.start - 1;
        // Find the word that contains the offset
        var found_1, len_1;
        line.text.replace(wordRe, (function (word, _, wordOffset) {
            if (wordOffset <= lineOffset_1 && wordOffset + word.length >= lineOffset_1 && word.match(special)) {
                found_1 = wordOffset;
                len_1 = word.length;
            }
        }));
        if (found_1 != null) {
            return lsp.Range.create(line.line - 1, found_1, line.line - 1, found_1 + len_1);
        }
    }
}
function insertTextOf(completion) {
    switch (completion.kind) {
        case 'attribute':
        case 'html attribute':
            return completion.name + "=";
    }
    return completion.name;
}
// This handler provides the initial list of the completion items.
connection.onCompletion(function (params) {
    var position = params.position, textDocument = params.textDocument;
    var _a = documents.getServiceInfo(textDocument, position), fileName = _a.fileName, service = _a.service, offset = _a.offset, languageId = _a.languageId;
    if (fileName && service && offset != null) {
        var result = service.getCompletionsAt(fileName, offset);
        if (!result) {
            return;
        }
        if (Array.isArray(result)) { // old ng.Completion[]
            if (languageId == 'html') {
                // The HTML elements are provided by the HTML service when the text type is 'html'.
                result = result.filter(function (completion) { return completion.kind != 'element'; });
            }
            var replaceRange_1 = getReplaceRange(params.textDocument, offset);
            return result.map(function (completion) { return ({
                label: completion.name,
                kind: compiletionKindToCompletionItemKind(completion.kind),
                detail: completion.kind,
                sortText: completion.sort,
                textEdit: insertionToEdit(replaceRange_1, insertTextOf(completion)),
                insertText: insertTextOf(completion)
            }); });
        }
        var entries = result.entries;
        return entries.map(function (entry) {
            var item = lsp.CompletionItem.create(entry.name);
            item.kind = compiletionKindToCompletionItemKind(entry.kind);
            item.detail = entry.kind;
            item.sortText = entry.sortText;
            item.textEdit = lsp.TextEdit.insert(position, entry.name);
            return item;
        });
    }
});
function ngDefinitionToDefinition(definition) {
    var locations = definition.map(function (d) {
        var document = lsp.TextDocumentIdentifier.create(documents_1.fileNameToUri(d.fileName));
        var positions = documents.offsetsToPositions(document, [d.span.start, d.span.end]);
        return { document: document, positions: positions };
    }).filter(function (d) { return d.positions.length > 0; }).map(function (d) {
        var range = lsp.Range.create(d.positions[0], d.positions[1]);
        return lsp.Location.create(d.document.uri, range);
    });
    if (locations && locations.length) {
        return locations;
    }
}
function logErrors(f) {
    try {
        return f();
    }
    catch (e) {
        if (e.message && e.stack)
            connection.console.error("SERVER ERROR: " + e.message + "\n" + e.stack);
        throw e;
    }
}
connection.onDefinition(function (params) { return logErrors(function () {
    var textDocument = params.textDocument, position = params.position;
    var _a = documents.getServiceInfo(textDocument, position), fileName = _a.fileName, service = _a.service, offset = _a.offset;
    if (fileName && service && offset != null) {
        var result = service.getDefinitionAt(fileName, offset);
        if (!result) {
            return;
        }
        if (Array.isArray(result)) {
            // Backwards compatibility with old ng.Location[] (ng.Definition)
            return ngDefinitionToDefinition(result);
        }
        var _b = result, textSpan = _b.textSpan, definitions = _b.definitions;
        if (!definitions || !definitions.length) {
            return;
        }
        var _c = documents.offsetsToPositions(textDocument, [
            textSpan.start,
            textSpan.start + textSpan.length,
        ]), start = _c[0], end = _c[1];
        var originSelectionRange_1 = lsp.Range.create(start, end);
        return definitions.map(function (d) {
            var targetUri = lsp.TextDocumentIdentifier.create(documents_1.fileNameToUri(d.fileName));
            var _a = documents.offsetsToPositions(targetUri, [
                d.textSpan.start,
                d.textSpan.start + d.textSpan.length,
            ]), start = _a[0], end = _a[1];
            var targetRange = lsp.Range.create(start, end);
            return {
                originSelectionRange: originSelectionRange_1,
                targetUri: targetUri.uri,
                targetRange: targetRange,
                targetSelectionRange: targetRange,
            };
        });
    }
}); });
function ngHoverToHover(hover, document) {
    if (hover) {
        var positions = documents.offsetsToPositions(document, [hover.span.start, hover.span.end]);
        if (positions) {
            var range = lsp.Range.create(positions[0], positions[1]);
            return {
                contents: { language: 'typescript', value: hover.text.map(function (t) { return t.text; }).join('') },
                range: range
            };
        }
    }
}
connection.onHover(function (params) { return logErrors(function () {
    var position = params.position, textDocument = params.textDocument;
    var _a = documents.getServiceInfo(textDocument, position), fileName = _a.fileName, service = _a.service, offset = _a.offset;
    if (fileName && service && offset != null) {
        var result = service.getHoverAt(fileName, offset);
        if (!result) {
            return;
        }
        if (isNgHover(result)) {
            // Backwards compatibility with old ng.Hover
            return ngHoverToHover(result, params.textDocument);
        }
        var _b = result, kind = _b.kind, kindModifiers = _b.kindModifiers, textSpan = _b.textSpan, displayParts = _b.displayParts, documentation = _b.documentation;
        var desc = kindModifiers ? kindModifiers + ' ' : '';
        if (displayParts) {
            // displayParts does not contain info about kindModifiers
            // but displayParts does contain info about kind
            desc += displayParts.map(function (dp) { return dp.text; }).join('');
        }
        else {
            desc += kind;
        }
        var contents = [{
                language: 'typescript',
                value: desc,
            }];
        if (documentation) {
            for (var _i = 0, documentation_1 = documentation; _i < documentation_1.length; _i++) {
                var d = documentation_1[_i];
                contents.push(d.text);
            }
        }
        var _c = documents.offsetsToPositions(textDocument, [
            textSpan.start,
            textSpan.start + textSpan.length,
        ]), start = _c[0], end = _c[1];
        return {
            contents: contents,
            range: lsp.Range.create(start.line - 1, start.character - 1, end.line - 1, end.character - 1),
        };
    }
}); });
function isNgHover(result) {
    if (result.span && Array.isArray(result.text)) {
        return true;
    }
    return false;
}
// Listen on the connection
connection.listen();
//# sourceMappingURL=server.js.map